
//  file save added
// Using the DF robot DFR-0654  Firebeetle 2
// this version plots the data as soon as its collected and pauses for a few seconds at the end of sweep to let the phone plot catch up to the data.
// now uses the MAX30003 INT pin to synchronize sample collection instead of reading the status register repeatedly


#include <SPI.h>
#include "protocentral_max30003.h"
#include "BluetoothSerial.h"

#define MAX30003_CS_PIN D6
#define NUM_SAMPLES 512
#define MAX30003_INT_PIN D3

MAX30003 max30003(MAX30003_CS_PIN);

// incoming data buffer
String in;
bool Start=false;
 int n=0;
 int m;
 char BTbuffer[100];
 int ECGval[NUM_SAMPLES]; 
 int y;

 uint8_t statusReg[3];
 uint8_t q;

 uint8_t buffer[8];
 int16_t ecgval=0;
 uint8_t temp;
 int32_t ecg;
 int16_t Filecount=0;
 volatile bool rtorFlag = false;
 int tick=0;

BluetoothSerial btSerial;

hw_timer_t *Timer0_Cfg = NULL;

void IRAM_ATTR Timer0_ISR()
{
    digitalWrite(D9, !digitalRead(D9));
}

void setup()
{
    pinMode(D9,OUTPUT);
    pinMode(D3,INPUT_PULLUP);
    digitalWrite(D9,LOW);   // turn off heart rate LED
    //set up the timer used to trigger LED at heart rate
    Timer0_Cfg = timerBegin(0, 40000, true);
    timerAttachInterrupt(Timer0_Cfg, &Timer0_ISR, true);
    timerAlarmDisable(Timer0_Cfg);
   
    pinMode(MAX30003_CS_PIN,OUTPUT);
    digitalWrite(MAX30003_CS_PIN,HIGH); //disable MAX30003 device
    Serial.begin(115200); 
    SPI.begin();
    bool ret = max30003.max30003ReadInfo();
    if(ret){
      Serial.println("Max30003 read ID Success");
    }
    else{
      while(!ret){
        //loop here untill the issue is fixed.
        ret = max30003.max30003ReadInfo();
        Serial.println("Failed to read MAX30003 ID");
        delay(5000);
      }
    }

    btSerial.begin("ECG");
    Serial.println("Bluetooth ready to pair!");
    max30003.max30003RegRead(STATUS, statusReg);
}

void loop()
{
  bool q;
   while(btSerial.available()) {
    const char c = btSerial.read();
    in += c;
    if(c == '\n') {
      // parse message string
      parseGuioMsg(in);
      // clear buffer
      in = "";
    }  
  }

 if (Start == true) {
 
   max30003.max30003Synch();   // reset FIFO pointers
  for (int j=0;j<NUM_SAMPLES;j++) {

     // wait til the rising edge of INT pin indicating a sample is ready 
    do {
       q=digitalRead(MAX30003_INT_PIN);
    } while (q == 1);
    do {
     q = digitalRead(MAX30003_INT_PIN);
    } while (q == 0);

    max30003.max30003RegRead(STATUS, statusReg);
    if(statusReg[1] & RTOR_INTR_MASK) {
      rtorFlag=true;
      }

    max30003.max30003ReadData(1,buffer);  // retrieve 1 sample
    ecgval= buffer[0] <<10;
    ecgval+= buffer[1]<<2;
    ecgval+= buffer[2]>>6;
    y =512 + ecgval/64;
    ECGval[j]=y;
    sprintf(BTbuffer," @ch0 PLI:\"id0\" YP:\"%4d\" \r\n" ,y); 
    sendMsg(BTbuffer);
  }
   
   
 
   if(rtorFlag){
        rtorFlag = false;
        max30003.getHRandRR();   //It will store HR to max30003.heartRate and rr to max30003.RRinterval.
        Serial.print("Heart Rate  = ");
        int hr =max30003.heartRate;
        Serial.println(hr);
        sprintf(BTbuffer,"@lb1 X:23 Y:70 FSZ:6 TXT:\"%3d\" \r\n" ,hr); 
        sendMsg(BTbuffer);
        Serial.print("RR interval  = ");
        int RRint= max30003.RRinterval;
        Serial.println(RRint);
        // flash the LED at the heart rate
        timerAlarmWrite(Timer0_Cfg, RRint, true);
        timerAlarmEnable(Timer0_Cfg);
      } 
      delay(5000);  // let the chart catch up weith the incomeing samples
      tick++;
      if ((tick > 4) && (rtorFlag == false)) {
        timerAlarmDisable(Timer0_Cfg);
        digitalWrite(D9,LOW);   // turn off heart rate LED
        tick=0;
      } 
     }
 } 


void sendMsg(const String &msg) {
  btSerial.write((const uint8_t*) msg.c_str(), msg.length());
}

void parseGuioMsg(const String &msg) {
  Serial.println(msg);
  if(msg.startsWith("@init")) {
    Serial.println("GUI-O app is requesting INITIALIZATION!");
    
    // clear screen and set background
    sendMsg("@cls\r\n");
    sendMsg("@guis BGC:#FFFFFF ASR:0.4497 \r\n");
    sendMsg("|SORI UID:sori HID:sori ORI:2 \r\n");   // landscape
    delay(1000); //  *********This is important- landscape mode won't work without it !!!   *****************************


    // initialize simple example GUI
sendMsg("|CH UID:ch0 X:50 Y:35 W:98 H:60 FSZ:4 CHT:2 CHN:\"ECG\" XLO:0 XHI:512 YLO:0 YHI:1024 SHG:0 SHYA:1  SHVL:0 SHSL:0 DRA:0 XMA:8.0 BSZ:512 BMA:0.05 XASC:0 YASC:0 YTC:0 XMA:0 \r\n");
sendMsg("|LB UID:lb0 X:10 Y:70 W:25 FSZ:6 TXT:\"Heart Rate\"\r\n");
sendMsg("|LB UID:lb1 X:23 Y:70 FSZ:6 TXT:\"---\"\r\n");
sendMsg("|BT UID:bt1 X:12 Y:90 W:15 H:10 BGC:#FD4C4C SBGC:#FF4C4C FGC:#F9000000 FSZ:7 TXT:\"Test\"\r\n");
sendMsg("|BT UID:bt2 X:35 Y:90 W:20 H:10 BGC:#4CFF4C SBGC:#4CFF4C FGC:#000000 FSZ:7 TXT:\"Operate\"\r\n");
sendMsg("|BT UID:bt3 X:60 Y:90 W:20 H:10 BGC:#92F4FC SBGC:#92F4FC FGC:#000000 FSZ:7 TXT:\"Save\"\r\n");
int y;

for (int i=0;i<5;i++){
  y=512;
  sprintf(BTbuffer," @ch0 PLI:\"id0\" PLC:\"#000000\"  YP:\"%5d\" \r\n" ,y); 
  sendMsg(BTbuffer);
  }
 n=5;
 max30003.max30003Begin();
 Serial.println("operate mode");
 Start=true; 
}
 
 else if(msg.startsWith("@bt1")) {
   max30003.max30003Calibrate(); 
   Serial.println("Test mode");
   delay(2000);
  } 

else if(msg.startsWith("@bt2")) {
  max30003.max30003Begin();
  Serial.println("operate mode");
  delay(2000);
  } 

   else if(msg.startsWith("@bt3")) {
    Serial.println("File save mode");
    sendMsg("|EXTF UID:extf1 HID:extf FNA:\"ECG.txt\"\r\n");
    sendMsg("@extf1 FAC:0 \r\n");
    sendMsg("@extf1 FAC:1 FP:\"ECG Data\"\r\n");
    for (int j=0;j< NUM_SAMPLES;j+=8) {
      y=ECGval[j];
      sprintf(BTbuffer,"@extf1 DTF:\"\" FP:\"%4d,%4d,%4d,%4d,%4d,%4d,%4d,%4d\"\r\n" ,ECGval[j],ECGval[j+1],ECGval[j+2],ECGval[j+3],ECGval[j+4],ECGval[j+5],ECGval[j+6],ECGval[j+7]);  
      sendMsg(BTbuffer);
      
    }
    sendMsg("@extf1 FAC:3 \r\n");

  } 
}

